package rooms

import (
	"clickandjoin.app/managementsystem/modules/scylladb"
	"clickandjoin.app/managementsystem/modules/structs"
	gocnjhelper "git.clickandjoin.umbach.dev/ClickandJoin/go-cnj-helper"
	"git.clickandjoin.umbach.dev/ClickandJoin/go-cnj-helper/dbstructs"
	"github.com/gofiber/fiber/v2"
)

func GetAllRooms(c *fiber.Ctx) error {
	// swagger:operation GET /rooms rooms roomsGetAllRooms
	// ---
	// summary: List of rooms
	// consumes:
	// - application/json
	// produces:
	// - application/json
	// responses:
	//  '200':
	//    schema:
	//      "$ref": "#/definitions/RoomsResponse"
	//  '500':
	//    description: Internal server error

	var rooms []dbstructs.Room

	q := scylladb.Session.Query(gocnjhelper.DbMRooms.SelectAll())

	if err := q.SelectRelease(&rooms); err != nil {
		gocnjhelper.LogErrorf("Failed to get rooms, err: %s", err)
	}

	return c.JSON(structs.RoomsResponse{Rooms: rooms})
}

func GetAllRoomUsers(c *fiber.Ctx) error {
	// swagger:operation GET /rooms/users rooms roomsGetAllUsers
	// ---
	// summary: List all room users
	// consumes:
	// - application/json
	// produces:
	// - application/json
	// responses:
	//  '200':
	//    schema:
	//      "$ref": "#/definitions/RoomUsersResponse"
	//  '500':
	//    description: Internal server error

	var roomUsers []dbstructs.RoomUsers

	q := scylladb.Session.Query(gocnjhelper.DbMRoomUsers.SelectAll())

	if err := q.SelectRelease(&roomUsers); err != nil {
		gocnjhelper.LogErrorf("Failed to get room users, err: %s", err)
	}

	return c.JSON(structs.RoomUsersResponse{RoomUsers: roomUsers})
}
